/*
 * (C) Copyright Pattern Recognition Group Delft. All rights reserved.
 * 
 * NRUTIL : Utilities of the Numerical Recipes in C.
 * 
 * References :Press W, Flannery B, Teukolsky S, Vetterling W, "Numerical
 * Recipes in C", Cambridge University Press, 1988. See chapter 1.
 * 
 * Changes : All floats are replaced by doubles.
 * 
 * Made by : W.F. Schmidt 
 * SccsId  : %Z%$RCSfile: nrutil.c,v $ V%Q% $Revision: 2.5 $ $Date: 1996/10/07 06:50:00 $
 */

#ifndef TCDOS
#include <malloc.h>
#else
#pragma warn -par
#include <alloc.h>
#endif
#include <stdio.h>

#define NRUTIL
#include <sprlib.h>

void nrerror (char error_text[])
{
   fprintf(stdout, "\nNRinC - ERROR in numerical library : ");
   fprintf(stdout, "%s\n", error_text);
	sprerror("Numerical recipes in C reports SPRLIB about error."); 	
}

int *ivector (int nl, int nh)
{
   int *v;

   v = (int *) malloc((unsigned) (nh - nl + 1) * sizeof(int));
   if (!v)
   {
      nrerror("allocation failure in ivector()");
      return NULL;
   }
   return v - nl;
}

/*
 * This procedure creates a vector of characters (a string).
 */
char *svector (int nl, int nh)
{
   char *v;

   v = (char *) malloc((unsigned) (nh - nl + 1) * sizeof(char));
   if (!v)
   {
      nrerror("allocation failure in svector()");
      return NULL;
   }
   return v - nl;
}

double *vector (int nl, int nh)
{
   double *v;
 
   v = (double *) malloc((unsigned) (nh - nl + 1) * sizeof(double));
   if (!v)
   {
      nrerror("allocation failure in dvector()");
      return NULL;
   }
   return v - nl;
}

double **matrix (int nrl, int nrh, int ncl, int nch)
{
   int i;
   double **m;

   m = (double **) malloc((unsigned) (nrh - nrl + 1) * sizeof(double *));
   if (!m)
   {
      nrerror("allocation failure 1 in dmatrix()");
      return NULL;
   }
   m -= nrl;

   for (i = nrl; i <= nrh; i++)
   {
      m[i] = (double *) malloc((unsigned) (nch - ncl + 1) * sizeof(double));
      if (!m[i])
      {
         nrerror("allocation failure 2 in dmatrix()");
         return NULL;
      }
      m[i] -= ncl;
   }
   return m;
}

int **imatrix (int nrl, int nrh, int ncl, int nch)
{
   int i, **m;

   m = (int **) malloc((unsigned) (nrh - nrl + 1) * sizeof(int *));
   if (!m)
   {
      nrerror("allocation failure 1 in imatrix()");
      return NULL;
   }
   m -= nrl;

   for (i = nrl; i <= nrh; i++)
   {
      m[i] = (int *) malloc((unsigned) (nch - ncl + 1) * sizeof(int));
      if (!m[i])
      {
         nrerror("allocation failure 2 in imatrix()");
         return NULL;
      }
      m[i] -= ncl;
   }
   return m;
}

double **submatrix (double **a, int oldrl, int oldrh, int oldcl, int oldch, int newrl, int newcl)
{
   int i, j;
   double **m;

   m = (double **) malloc((unsigned) (oldrh - oldrl + 1) * sizeof(double *));
   if (!m)
   {
      nrerror("allocation failure in submatrix()");
      return NULL;
   }
   m -= newrl;

   for (i = oldrl, j = newrl; i <= oldrh; i++, j++)
      m[j] = a[i] + oldcl - newcl;

   return m;
}

void free_ivector (int *v, int nl, int nh)
{
   free((char *) (v + nl));
}

void free_svector (char *v, int nl, int nh)
{
   free(v + nl);
}

void free_vector (double *v, int nl, int nh)
{
   free((char *) (v + nl));
}

void free_matrix (double **m, int nrl, int nrh, int ncl, int nch)
{
   int i;

   for (i = nrh; i >= nrl; i--)
      free((char *) (m[i] + ncl));
   free((char *) (m + nrl));
}

void free_imatrix (int **m, int nrl, int nrh, int ncl, int nch)
{
   int i;

   for (i = nrh; i >= nrl; i--)
      free((char *) (m[i] + ncl));
   free((char *) (m + nrl));
}

void free_submatrix (double **b, int nrl, int nrh, int ncl, int nch)
{
   free((char *) (b + nrl));
}

double **convert_matrix (double *a, int nrl, int nrh, int ncl, int nch)
{
   int i, j, nrow, ncol;
   double **m;

   nrow = nrh - nrl + 1;
   ncol = nch - ncl + 1;
   m = (double **) malloc((unsigned) (nrow) * sizeof(double *));
   if (!m)
   {
      nrerror("allocation failure in convert_matrix()");
      return NULL;
   }
   m -= nrl;
   for (i = 0, j = nrl; i <= nrow - 1; i++, j++)
      m[j] = a + ncol * i - ncl;
   return m;
}

void free_convert_matrix (double **b, int nrl, int nrh, int ncl, int nch)
{
   free((char *) (b + nrl));
}

#ifdef TCDOS
#pragma warn .par
#endif

/*
 * FPRINTF_VECTOR : Print a vector in Numerical Recipes format to the file
 * pointed by stream. The vector mean[1..n] is a vector with fixed lower
 * bound and upper bound n.
 *
 * Return value : TRUE  If an error is detected during fileio.
 *                FALSE If no error is detected.
 */

int fprintf_vector (FILE *stream, int n, double *mean)
{
   int i;

   if (fprintf(stream, "V = (") == EOF)
   {
      sprerror("fprintf_vector(): Function fprintf returns EOF.");
      return TRUE;
   }
   for (i = 1; i < n; i++)
   {
      if (fprintf(stream, "% 8.4E, ", mean[i]) == EOF)
      {
         sprerror("fprintf_vector(): Function fprintf returns EOF.");
         return TRUE;
      }
   }
   if (fprintf(stream, "% 8.4E)\n", mean[n]) == EOF)
   {
      sprerror("fprintf_vector(): Function fprintf returns EOF.");
      return TRUE;
   }

   return FALSE;
}

/*
 * FPRINTF_MATRIX : Print a matrix in Numerical Recipes format to the file
 * pointed by stream. The matrix mat[1..n][1..m] is a matrix with fixed lower
 * bound and upper bounds n and m.
 *
 * Return value : TRUE  If an error is detected during fileio.
 *                FALSE If no error is detected.
 */

int fprintf_matrix (FILE *stream, int n, int m, double **mat)
{
   int i, j;

   if (fprintf(stream, "M = |") == EOF)
   {
      sprerror("fprintf_matrix(): Function fprintf returns EOF.");
      return TRUE;
   }
   for (i = 1; i < m; i++)
   {
      if (fprintf(stream, "% 8.4E ", mat[1][i]) == EOF)
      {
         sprerror("fprintf_matrix(): Function fprintf returns EOF.");
         return TRUE;
      }
   }
   if (fprintf(stream, "% 8.4E|\n", mat[1][m]) == EOF)
   {
      sprerror("fprintf_matrix(): Function fprintf returns EOF.");
      return TRUE;
   }

   for (j = 2; j <= n; j++)
   {
      if (fprintf(stream, "    |") == EOF)
      {
         sprerror("fprintf_matrix(): Function fprintf returns EOF.");
         return TRUE;
      }
      for (i = 1; i < m; i++)
      {
         if (fprintf(stream, "% 8.4E ", mat[j][i]) == EOF)
         {
            sprerror("fprintf_matrix(): Function fprintf returns EOF.");
            return TRUE;
         }
      }
      if (fprintf(stream, "% 8.4E|\n", mat[j][m]) == EOF)
      {
         sprerror("fprintf_matrix(): Function fprintf returns EOF.");
         return TRUE;
      }
   }

   return FALSE;
}

#undef NRUTIL

/*
 * End of Source
 */
